%% Estimate individual-level MNL models
% Function that takes in choice model data and executes modified maximum likelihood
% estimation for each individual in the sample assuming a flat prior
% Written by Bart Frischknecht, first version 29 March 2013, 
% Edits:
%
%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ModelParams=EstModsIndCLConjPrior(ChoiceDataF,NIND, S,J,Nparams,NameLabels, MyCLEstFile,alpha,indindexon)

%%%%%%%%%%%%  Perform model estimation for set of data files %%%%%%%%%%%

% This script generates *.out text files, each corresponding to the model
% estimation for each individual in the sample.  You can turn off the
% output to the screen by commenting out all of the "disp" statements in
% the "doitIL.m" file.

% This code calls the estimation routine created by Ken Train.  The mxlmsl
% file and the files it calls were all created by Ken Train and were
% downloaded from his website. The original files downloaded from Ken
% Train's website were converted from scripts into functions so that data
% could be passed easily

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Inputs 
% Variable          Dimension                       Units                   Description
% ChoiceDataF       string                                                  Name of matrix that contains the data set for individual i
% NIND              scalar                          individuals             Number of individuals in the data
% S                 scalar                          choice sets             Number of choice sets faced by each individual
% J                 scalar                          choice tasks            Number of choice tasks faced by each individual
% Stop              scalar                          -                       Index indicating the column where the attribute codes stop
% MyCLEstFile       handle                          -                       Handle giving the name for the estimation file 
% alpha             scalar                          -                       The prior weight value
% InDesign_eff      (S*J) x K matrix                -                       The effects coded version of the design matrix


% R                 scalar                                                  number of individual respondents in data set
% I                 scalar                                                  number of choice observations (not number of individuals) in a data set
% k                 scalar                                                  number of total rows in the data file
% N                 scalar                                                  number of draws to use per person in numerical simulation (should be 1 unless estimating mixed logit models) 

%% OUTPUTS
% Variable          Dimension                       Units                   Description
% ModelParams       NIND x K matrix                 -                       Matrix containing the estimated model parameters for each individual
% DataOut           .out file                       -                       NOTE: This is turned off. Name of output file where estimation results are stored


ModelParams=zeros(NIND,Nparams); %Initialize the model parameters matrix

DataIn=ChoiceDataF; %The full sample data
ChoiceTask=zeros(S*J,1);
% relabel the choice task index sequentially for each individual
for s=1:S
    ChoiceTask((s-1)*J+1:s*J,1)=s*ones(J,1);
end
% include the additional choice tasks that make up the notional sample
for s=S+1:S+S*J
    ChoiceTaskEx((s-S-1)*J+1:(s-S)*J,1)=s*ones(J,1);
end
% relabel the respondent number index and add in the new choice task index
for i=1:NIND
    DataIn((i-1)*S*J+1:i*S*J,1)=ones(S*J,1);
    DataIn((i-1)*S*J+1:i*S*J,2)=ChoiceTask;
end

% initialize the artificial choice vector for the notional sample
yExtra=zeros(S*J*J,1);
% call the notional sample individual 2. This is what allows us to weight
% the notional sample differently from the observed data using Ken Train's
% maximum likelihood estimation code
IndExtra=2*ones(S*J*J,1);
% Initialize the weights vector
WeightsInd=zeros(S*J+S*J*J,1);
% Weight of 1 for each observation in the observed data
WeightsInd(1:S*J,1)=1;
% Weight of alpha/J for each observation in the notional sample for an
% aggregate effect of R_prior=alpha
WeightsInd(S*J+1:S*J+S*J*J)=alpha/J;

%indicate the chosen alternative for each choice task in the notional
%sample
for j=1:J
    yExtra((j+(S*J*(j-1))):J:((S*J*(j-1))+S*J),1)=1;
end
DataCols=size(DataIn,2);
weightcol=DataCols+1;

%Assign choices to the notional sample choice tasks
    ExCS(:,3)=yExtra;
    %Number the pseudo-observation choice tasks
    ExCS(:,2)=ChoiceTaskEx;
    %Label the pseudo-observations as individual 2 in data set for purposes
    %of weighting the pseudo-observations different than the standard observations
    ExCS(:,1)=IndExtra;
    

for i=1:NIND
     ExCS(:,4:DataCols)=repmat(DataIn((i-1)*S*J+1:i*S*J,4:DataCols),J,1); % The way the code 
                                                 % is written it requires 
                                                 % each individual to complete the
                                                 % the same number of alternatives
                                                 % and choice sets. This
                                                 % can be relaxed with more 
                                                 % general coding
    %Concatanate the pseudo-observations onto the standard observations
    %Add vector of weights for standard observations and pseudo-observations
    DataInI=[[DataIn((i-1)*S*J+1:i*S*J,:);ExCS],WeightsInd];
   
    k=size(DataInI,1);   %total number of alternatives seen in all the choice sets
    
    DataOut=['DataOut',int2str(i),'.out'];
    I=S+S*J; %number of total choice sets seen in the sample
    R=2; %number of people    
    N=1;   %number of draws per person
    %%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%
    if indindexon==1
        i   % show individual index of estimations
    end
    %%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%
    
   
   ModelParams(i,:)=MyCLEstFile(DataInI,DataOut,R,I,k,N,Nparams,weightcol,NameLabels);
   
end
ModelParams=ModelParams';


