%% MMLEst_SingInd
% Function that executes model estimation and tests models against holdouts
% as indicated by the settings and setup files. 
% Written by Bart Frischknecht, first version 29 March 2013, 
% Edits:
%
%%%%%%%%%%%%%%%%%%%%%%%

%% INPUTS
% Variable          Dimension                       Units                   Description
% FileNamePrefix    string                          -                       The prefix of the data file containing the parameter estimates 
%                                                                           and other results
% MyRunSettingsFile pointer                         -                       Name of the settings file that specifies the estimation details
% MySetupFile       pointer                         -                       Name of the setup file that specifies the data and model details
% Runs              1 x R vector                    -                       The vector of indices corresponding to the estimation runs to 
%                                                                           be executed, where estimation runs are specified in MyRunSettingsFile
%%%%%%%%%%%%%%%%%%%%%%%

%% OUTPUTS
% Variable          Dimension                       Units                   Description
% alpha_rec             Runs x 1 vector                 -                   A vector that reports recommended prior weight for each simulation run
% RPS_rec               Runs x 1 matrix                 -                   The root predictive score at the recommended prior weight for each simulation run
% RPS_bar               Runs x 1 cell                   -                   The average root predictive score from V subsamples for each of Q prior weights tested
%                                                                           for each simulation run, so the entry in each cell is a 1 x Q vector
% RPS_bar_std           Runs x Q cell                   -                   Estimated standard deviation for the root predictive score from V subsamples for each 
%                                                                           of Q prior weights tested for each simulation run, so the entry in each cell is a 1 x Q vector
% ModelParams_rec       Runs x 1 cell                   -                   Cell containing the K x N matrix of N individual vectors of K model parameters 
% Data                  Structure                       -                   Structure containg the performance measures for in sample estimation
%                                                                           choice sets, in sample holdout choice sets, out of sample estimation
%                                                                           choice sets, and out of sample holdout choice sets
%%%%%%%%%%%%%%%%%%%%%


function [alpha_rec_all, RPS_rec, RPS_bar, RPS_bar_std, ModelParams_rec]=MMLEst_SingInd(MyRunSettingsFile, MySetupFile,Runs,FileNamePrefix)
%% Describe Data Set

% Initialize the output variables
TotRuns=length(Runs);
V=[];
alpha_rec_all=zeros(TotRuns,1);
RPS_rec=zeros(TotRuns,1);
RPS_bar=cell(TotRuns,1);
RPS_bar_std=cell(TotRuns,1);
ModelParams_rec=cell(TotRuns,1);

%Collect the simulation run settings
    NumRunSettings=MyRunSettingsFile();

for indexi=1:TotRuns
    Loop=Runs(indexi);
    clearvars -except V Loop indexi Runs ObsWeight FindWeightObj MyRunSettingsFile MySetupFile FileNamePrefix NumRunSettings alpha_rec_all RPS_rec RPS_bar RPS_bar_std ModelParams_rec
    % Global variable declaration %
    global  ModelParams ModelParamsSc ChoiceDataSampDes ProbObsSampDes
    
    %% Initialize problem description
    [HO, dFF, NameLabels, ProdASC, J, T, HoldoutAltSize, DrawHoldOut, ...
            ChoiceCol, Q, ProbMR, MarkRelCS, SimData, UtForm,...
            Params, FirstChoiceData, MyCLEstFile...
            ]=MySetupFile();
    Vnew=[];
    if Loop~=Runs(1) && NumRunSettings(Loop,2)==NumRunSettings(Runs(indexi-1),2)
    else
        V=Vnew;
    end
    
    %% Initialize run settings
    N=NumRunSettings(Loop,1);       % Total number of individuals in the data
    PriorType=NumRunSettings(Loop,2);   % Indicator variable designating the choice of prior 1: flat prior; 
                                        % 2=empirical Bayes prior; 3=flat prior with market relevant choice sets
    S=NumRunSettings(Loop,3);           % Number of choice tasks faced by each individual
    alpha=NumRunSettings(Loop,4);   % Guess for prior weight
    Vee=NumRunSettings(Loop,5);     % Number of replications to make for checking validity   
    PercSamp=NumRunSettings(Loop,6);% percentage of sample to retain for prior weight calibration
    L=NumRunSettings(Loop,7);       % The number of prior weight values to search, logarithmically centered around alpha
    alpha_max=NumRunSettings(Loop,8); % Maximum prior weight to consider
    Plot=NumRunSettings(Loop,9);    % indicator for turning on (Plot=1) or off (Plot=0) the result plots
    indindexon=NumRunSettings(Loop,10);  % indicator that turns on reporting an individual index during estimation. 
    if L>2
        alpha_l=zeros(L,1);
        l_space=(log(alpha_max)-log(alpha))/floor(L/2);
        l_min=log(alpha)-(log(alpha_max)-log(alpha));%+q_space;
        for l=1:L
            alpha_l(l,1)=exp((l-1)*l_space+l_min);
        end
        %alpha_q=exp(alpha_q);
    elseif L==1
        alpha_l=alpha;
    elseif L==2
        alpha_l=[alpha;alpha_max];
    end

            
    
    DRAWS=1;  %Number of different times to repeat the run, for example when you simulate
              %a sample and want to replicate draws from the simulated utility function 
    if PercSamp==0
      NSAMP=1;
    else
      NSAMP=Vee;%Number of different subsample populations to be drawn from 
                %sample for prior weight calibration based on random sampling 
                %of respondents into the holdout group
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Build subsamples
    % if PercSamp~=0 then subsamples are formed for validation, e.g. prior
    % weight calibration
    if PercSamp~=0
        ValHO=floor((N-HO)*PercSamp);       % Validation holdout size
        NModParams=Params-ProdASC;          % Number of model parameters to estimate excluding the outside good parameter
        ModelParams=zeros(NModParams+ProdASC,N-HO); ModelParamsSc=zeros(NModParams+ProdASC,N-HO); % Predefine Model Parameters Matrix
        % Initialize variables for storing outputs
        AvgFitsSampDes=zeros(DRAWS,20); AvgFitsSampHO=zeros(DRAWS,20);  AvgFitsHODes=zeros(DRAWS,20); AvgFitsHOHO=zeros(DRAWS,20);
        AvgRsqSimParams=zeros(DRAWS,Params);AvgmSimParams=zeros(DRAWS,Params);AvgbSimParams=zeros(DRAWS,Params); AvgRsq_bestSimParams=zeros(DRAWS,Params);
        h=zeros(DRAWS,1,Params);p=zeros(DRAWS,1,Params);KS=zeros(DRAWS,1,Params);VCompSort=zeros(ValHO*DRAWS*NSAMP,Params);ModParSort=zeros(N-HO-ValHO,Params);AllKSEach=zeros(Params,DRAWS);AggKS=zeros(DRAWS,1);AllKSEach_d=zeros(Params,DRAWS);AggKS_d=zeros(DRAWS,1);
        AvgDerr=zeros(DRAWS,1); Tavg=zeros(DRAWS,1); Sigma2=zeros(N-HO-ValHO,1); Sigma2avg=zeros(DRAWS,1);
        % Initialize variables for holdout and sample indices
        HOIndex=zeros(ValHO,NSAMP,DRAWS);
        HoldoutMem= false(N-HO,NSAMP,DRAWS);
        SampleMem= true(N-HO,NSAMP,DRAWS);
        % create indices for sub sample holdouts and estimation 
        for dd=1:DRAWS
            SampI=1:N-HO;
            for ss=1:NSAMP
                HOIndex(:,ss,dd)=randsample(SampI,ValHO);
                HoldoutMem(HOIndex(:,ss,dd),ss,dd)=true;
                SampleMem(HOIndex(:,ss,dd),ss,dd)=false;
                %SampRem=ismember(SampI(1,:),HOIndex(:,ss,dd));
                %SampI(SampRem)=[];
            end
        end
    else
        ValHO=HO;                       % Validation holdout size
        NModParams=Params-ProdASC;      % Number of model parameters to estimate excluding the outside good parameter
        ModelParams=zeros(NModParams+ProdASC,N-HO); ModelParamsSc=zeros(NModParams+ProdASC,N-HO); % Predefine Model Parameters Matrix
        % Initialize variables for storing outputs
        AvgFitsSampDes=zeros(DRAWS,20); AvgFitsSampHO=zeros(DRAWS,20);  AvgFitsHODes=zeros(DRAWS,20); AvgFitsHOHO=zeros(DRAWS,20);
        AvgRsqSimParams=zeros(DRAWS,Params);AvgmSimParams=zeros(DRAWS,Params);AvgbSimParams=zeros(DRAWS,Params); AvgRsq_bestSimParams=zeros(DRAWS,Params);
        h=zeros(DRAWS,1,Params);p=zeros(DRAWS,1,Params);KS=zeros(DRAWS,1,Params);VCompSort=zeros(ValHO*DRAWS*NSAMP,Params);ModParSort=zeros(N-HO,Params);AllKSEach=zeros(Params,DRAWS);AggKS=zeros(DRAWS,1);AllKSEach_d=zeros(Params,DRAWS);AggKS_d=zeros(DRAWS,1);
        AvgDerr=zeros(DRAWS,1); Tavg=zeros(DRAWS,1); Sigma2=zeros(N-HO,1); Sigma2avg=zeros(DRAWS,1);
        %This section of code identifies the respondent numbers for the different
        %subsamples.  If DrawHoldout==0 then in-sample
        %respondents should have respondent id from 1:N-HO, and the out of sample
        %respondents should have id from N-HO+1:N; Otherwise
        %subsamples are chosen randomly from full sample
        if DrawHoldOut==0
                % Initialize variables for holdout and sample indices
                HOIndex=zeros(HO,NSAMP,DRAWS);
                HoldoutMem= false(N,NSAMP,DRAWS);
                SampleMem= true(N,NSAMP,DRAWS);
                % create indices for sub sample holdouts and estimation
                if HO~=0
                    for dd=1:DRAWS
                        for ss=1:NSAMP
                            HOIndex(:,ss,dd)=N-HO+1:N;
                            HoldoutMem(HOIndex(:,ss,dd),ss,dd)=true;
                            SampleMem(HOIndex(:,ss,dd),ss,dd)=false;
                        end
                    end
                end
        elseif DrawHoldout==1
                % Initialize variables for holdout and sample indices
                HOIndex=zeros(HO,NSAMP,DRAWS);
                HoldoutMem= false(N,NSAMP,DRAWS);
                SampleMem= true(N,NSAMP,DRAWS);
                % create indices for sub sample holdouts and estimation
                for dd=1:DRAWS
                    for ss=1:NSAMP
                        HOIndex(:,ss,dd)=randsample(N,HO,true);
                        HoldoutMem(HOIndex(:,ss,dd),ss,dd)=true;
                        SampleMem(HOIndex(:,ss,dd),ss,dd)=false;
                    end
                end
        end
    end

    %% Run estimation and prior weight calibration 
    %Call the procedure to estimate individual models and evaluate validation subsamples
    [alpha_rec_all(Loop,1) ]=FindOptWeight();
    
end

function [alpha_rec]=FindOptWeight()

 
    for d=1:DRAWS
        

        if SimData==0;
         ChoiceData=load(FirstChoiceData,'-ascii');
        elseif SimData==1;
            
        end
        clear HoldoutMem
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     RPS=zeros(L,NSAMP);
     ModelParams_st=cell(L,1);
    for l=1:L
        %% Separate holdout choice tasks from estimation choice tasks
        ChoiceDataOutSamp=ChoiceData(ChoiceData(:,4)==1,:);
        ChoiceDataInSamp=ChoiceData(ChoiceData(:,4)~=1,:);

        %% Estimate Models
            %Nparams determined by the format of the ChoiceDataSampDes
            %matrix and is related to the number of attribute levels to be
            %estimated
            if PercSamp==0
                ValHO=0;
            end
            Nparams=Params;
            if PriorType==1
             ModelParams=EstModsIndCLConjPrior(ChoiceDataInSamp(:,[1,2,5,9:(8+Nparams)]),N-HO, S,J,Nparams,NameLabels,MyCLEstFile,alpha_l(l,1),indindexon);
             ModelParams_st{l}=ModelParams;
            elseif PriorType==2
             ModelParams=EstModsIndCLConjPriorIndepend(ChoiceDataInSamp(:,[1,2,5,9:(8+Nparams)]),N-HO, S,J, Nparams,NameLabels,MyCLEstFile,alpha_l(l,1),indindexon);
             ModelParams_st{l}=ModelParams;
            elseif PriorType==3
             ModelParams=EstModsIndCLConjPriorMarkRel(ChoiceDataInSamp(:,[1,2,5,9:(8+Nparams)]),N-HO,S,J,Nparams,NameLabels, MyCLEstFile,alpha_l(l,1),indindexon,Q, ProbMR, MarkRelCS);
             ModelParams_st{l}=ModelParams;
            end


        %% Calculate individual predicted shares for In Sample Design Sets
            ProbObsSampDes=ChoiceDataInSamp(1:N*S*J,ChoiceCol);

            VInSamp=zeros((N-HO)*S*J,N-HO);
            % Compute systematic utility
            for eye=1:N-HO
            for i=1:N-HO
                VInSamp((i-1)*S*J+1:i*S*J,eye)=ChoiceDataInSamp((i-1)*S*J+1:i*S*J,9:8+Nparams)*ModelParams(:,eye);
            end
            end
            % Compute logit probabilities
            ExpVInSamp=exp(VInSamp);
            clear VInSamp
            SumExpVInSamp=zeros((N-HO)*S*J,eye);
            for eye=1:N-HO
            for i=1:N-HO
                for s=1:S                
                    SumExpVInSamp((i-1)*S*J+(s-1)*J+1:(i-1)*S*J+(s-1)*J+J,eye)=sum(ExpVInSamp((i-1)*S*J+(s-1)*J+1:(i-1)*S*J+(s-1)*J+J,eye),1);
                end
            end
            end
            ProbPredInSamp=ExpVInSamp./SumExpVInSamp;
            clear SumExpVSampDes ExpVSampDes
            
            ProbObsSampDes_mat=repmat(ProbObsSampDes,1,N);
            loglike_InSamp=log(ProbPredInSamp.^ProbObsSampDes_mat);
             
            for i=1:N-HO
                ProbPredSampDes((i-1)*S*J+1:i*S*J,1)=ProbPredInSamp((i-1)*S*J+1:i*S*J,i);
            end
            
        %% Calculate root predictive score for validation subsample 
        % For a given validation sample calculate the average likelihood for a new individual h
        % using the predictions of each of the estimation samples
        T_like=zeros(ValHO,NSAMP);
        for nsamp=1:NSAMP
            SampIndex=setdiff(1:N-HO,HOIndex(:,nsamp,d));
            for h=1:ValHO
                T_like(h,nsamp)=1/(N-ValHO)*sum(exp(sum(loglike_InSamp((HOIndex(h,nsamp,d)-1)*S*J+1:HOIndex(h,nsamp,d)*S*J, SampIndex),1)),2);
            end
        end
        RPS(l,:)=exp(sum(log(T_like),1)/(S*ValHO));
        RPS_bar{Loop,1}(l,1)=1/NSAMP*sum(RPS(l,:),2);
        
    end  
        RPS_bar_std{Loop,1}(1:L,1)=std(RPS,0,2);
        
        %% Plot results %%%%%%%%%
        [RPS_rec(Loop,1), maxRPS_Ix]=max(RPS_bar{Loop,1});
        alpha_rec=alpha_l(maxRPS_Ix,1);
        ModelParams_rec{Loop,1}=ModelParams_st{maxRPS_Ix};
        
        %%
        if Plot==1
            figure(Loop)
            clf
            %subplot(2,2,1)
            semilogx(alpha_rec,RPS_rec(Loop,1),'rx');ylabel('RPS'); xlabel('R_{prior}');
            hold on
            errorbar(alpha_l,RPS_bar{Loop,1},RPS_bar_std{Loop,1},'ko-')
            title('Root predictive score vs. prior weight');
            axis([alpha_l(1) alpha_l(L) 0 RPS_rec(Loop,1)+.1]);
            hold off
        end
 
 % plot rps and std dev.

 % Save parameters for best solution
 % display plot for best solution
 
 
    end

%% Save outputs to data file     
Data.alpha_rec=alpha_rec;
Data.RPS=RPS;
Data.RPS_rec=RPS_rec(Loop,1);
Data.RPS_bar=RPS_bar{Loop,1};
Data.RPS_bar_std=RPS_bar_std{Loop,1};
Data.ModelParams_rec=ModelParams_rec{Loop,1};

RunNum=num2str(Loop);
FileName=strcat(FileNamePrefix, RunNum);
save(FileName,'Data');

end


end 