%% Individual level choice model estimation software
% 
% Matlab implementation to estimate logit models for single individuals 
% using a conjugate prior modified maximum likelihood method
% written by Bart Frischknecht
% www.censoc.uts.edu.au/resources
% Version 1, 29 April 2013
% 
%% Summary
% This software implements a method for estimating logit choice models for 
% small sample data, including single individuals, that is computationally 
% simpler and relies on weaker prior distributional assumptions compared to 
% hierarchical Bayes estimation. The method is presented in this working 
% paper <link: www.censoc.uts.edu.au/researchoutput/wp11-007.pdf>. 
% 
%% Data format
% The software assumes a data structure where there are multiple 
% observations for each respondent, each respondent faces the same number 
% of choice sets, and each choice set contains the same number of 
% alternatives. The data are clustered sequentially for each respondent 
% where each row represents a single alternative in a single choice set for
% a single respondent. The data should be prepared according to the 
% following format. 
% Column 1: responseid  -   sequentially numbered respondents 1,...,N
% Column 2: subset      -   sequentially numbered choice tasks 1,...,S
% Column 3: alt         -   sequentially numbered alternatives 1,...,J	
% Column 4: holdout_indic   -   entry should equal 1 if the alternative is 
%                               to be witheld from estimation
% Column 5: choice      -   binary variable {0,1} with 1 indicating chosen
%                           alternative
% Column 6: most        -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 7: least       -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 8: Rank        -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 9 through 9+K: -   Design matrix where each row lists the effects 
%                           coded design variables for each alternative 
% 
% 
%% Getting started
% The matlab script file RunChoiceModelEstimation.m executes the estimation
% procedure. "RunChoiceModelEstimation" can be called at the command prompt
% to execute the procedure using a sample of car insurance choice data. The
% procedure saves a *.mat file with the results and prints a figure to the
% screen that shows the average root predictive score for each value of the
% prior weight tested.
%
% The input files can be modified and the data file replaced in order to
% apply the estimation procedure to a new data set. 
% 
% The 'MyRunSettingsFile'allows multiple estimation runs to be defined
% simultaneously as in a batch mode. Different runs may use different 
% priors, test different prior weights, change the validation subsample 
% size, etc. The 'Runs' vector in the 'RunChoiceModelEstimation' file 
% indicates the estimation runs from 'MyRunSettingsFile' to be executed. 
%
%% Inputs
% The user provides the inputs for the estimation procedure by modifying 
% the RunChoiceModelEstimation.m script file and four additional input 
% files. 
%
% %%% RunChoiceModEst.m %%%
% Inputs                Description
% OutputFileNamePrefix  The prefix of the data file containing the
%                       estimates fo the parameters and the other results
% MyRunSettingsFile     The name of the settings file that specifies the 
%                       estimation details
% MySetupFile           The name of the setup file that specifies the data 
%                       and model details
% Runs                  A 1 x R vector of indices corresponding to the 
%                       estimation runs to be executed, where estimation 
%                       runs are defined in MyRunSettingsFile 
%
% %%% MyRunSettingsFile          %%%
% Inputs                Description
% N                     Number of individuals in the sample
% PriorType             An index identifying the type of conjugate prior to 
%                       use in estimation: 1=flat prior; 2=empirical Bayes 
%                       prior; 3=flat prior with market relevant choice 
%                       sets
% S                     Number of choice sets faced by each individual
% alpha_guess           Guess for relative weight of the prior compared to 
%                       the original data
% Vee                   The number of subsample validation replications for 
%                       a particular prior weight ratio
% PercSamp              The percentage of the sample randomly drawn as the 
%                       validation subsample for a particular replication 
%                       vee=1, ..., Vee
% L                     The number of prior weight values to search. Prior
%                       weight values are logarithmically centered around 
%                       alpha_guess
% alpha_max             The maximum prior weight to test. Note, when L is 
%                       odd and greater than 1, then alpha_max will be 
%                       included in the prior weights tested. When L is 
%                       even then the largest alpha tested will be smaller 
%                       than alpha_max
% Plot                  An indicator that turns plotting on (Plot=1) or 
%                       turns plotting off (Plot=0)
% indindexon            An indicator that turns on reporting an individual 
%                       index during estimation. This allows the user to 
%                       observe the progress through the individual 
%                       estimations for each prior weight value tested

%
% %%% MySetupFile                %%%
% Inputs                Description
% HO                    Number of people from total sample (N) to holdout 
%                       from estimation. This variable does not refer to
%                       the individuals used in the validation subsamples.
%                       Instead, this number should be set to the desired 
%                       number of respondents for a true holdout sample to  
%                       be used for later testing. The data from the 
%                       holdout individuals are excluded from all 
%                       estimation and prior weight calibration
% dFF                   A vector that describes the model parameters. The 
%                       number of entries corresponds to the number of 
%                       attributes in the model. Each entry corresponds to 
%                       the number of levels of the particular attribute.
%                       Attributes should be listed in the order they 
%                       appear in the data
% NAMESF                A K x 1 cell array of strings containing the names
%                       for each model parameter. Typically these 
%                       correspond to an attribute and attribute level 
%                       identification
% ProdASC               ProdASC=1 if a parameter for "no choice" or 
%                       "status quo" is to be estimated. This parameter 
%                       should not be included in the dFF vector 
% J                     Number of alternatives per choice set faced by 
%                       each individual
% T                     Number of holdout choice sets per person. This
%                       number should be set to the number of choice sets 
%                       that are intended to be used for later testing. 
%                       These choice sets are not included in any 
%                       estimation or calibration procedure
% HoldoutAltSize        The total number of alternatives per individual
%                       that are included in the holdout choice sets
% DrawHoldOut           DrawHoldout=1 to select HO individuals for holdout 
%                       sample randomly from the original sample 
%                       DrawHoldout=0 to select HO individuals from the 
%                       bottom of the original data file. The value of this 
%                       parameter does not matter when HO=0
% ChoiceCol             Column in 'FirstChoiceData' that has the choice 
%                       indicator ({0,1} with 1 indicating chosen 
%                       alternative)
% Q                     The number of market relevant choice tasks included 
%                       in the notional sample when the market relevant 
%                       prior is used
% ProbMR                The choice probability assigned to each alternative 
%                       in each market relevant choice task
% MarkRelCS             Coded design matrix for the market relevant choice 
%                       sets where each row represents a particular 
%                       alternative in a particular market relevant choice 
%                       set
% FirstChoiceData       The name of the data file that contains the choice 
%                       data including the effects coded attributes of each 
%                       alternative. The format for the data is described
%                       above and again in 'MySetupFile'
% MyCLEstFile           The file name for the file that executes maximum 
%                       likelihood estimation
%
% Note: The following two parameters are legacy from a prior software
% implementation. These parameters should be left at the default values.
% SimData               To generate simulated data, SimData=1; Otherwise, 
%                       estimation expects to load data file. 
% UtForm                The distribution of the utility function for 
%                       simulated data
%
% %%% MyCLEstFile                %%%
% This file and the files it calls are modifications of the Matlab version
% of the mixed logit simulated maximum likelihood software by Kenneth Train
% available on his website at http://elsa.berkeley.edu/~train/software.html 
% The primary modification to Train's code is that some script files have 
% been turned into functions in order to facilitate passing parameter 
% values and results between files.
%
% Lines 33-35 can be uncommented in order to turn on the log file. Turning
% on the log file will write a text file for each individual estimation.
% This estimation file will be overwritten each time a new prior weight is
% used. This means that after a given estimation run, the text files
% reporting the estimation results will correspond only to the last prior
% weight value (alpha) tested--not the recommended prior weight value
%
% You can adjust the maximum likelihood estimation parameters if you are
% having numerical convergence difficulties with a particular data set. The
% parameters available in the 'MyCLEstFile' are MAXITERS to specify the
% maximum number of optimization iterations, PARAMTOL to specify the
% maximum change in the values of the parameter estimates for convergence, 
% and LLTOL to specify the change in the log-likelihood (objective
% function) for convergence
%
% Inputs                Description
% 
% %%% FirstChoiceData            %%%       
% Input                 Description
% Data                  The choice data including the effects coded 
%                       attributes of each alternative. The file should be
%                       a tab-delimited text file. The data should be 
%                       prepared according to the following format. 
% Column 1: responseid  -   sequentially numbered respondents 1,...,N
% Column 2: subset      -   sequentially numbered choice tasks 1,...,S
% Column 3: alt         -   sequentially numbered alternatives 1,...,J	
% Column 4: holdout_indic   -   entry should equal 1 if the alternative is 
%                               to be witheld from estimation
% Column 5: choice      -   binary variable {0,1} with 1 indicating chosen
%                           alternative
% Column 6: most        -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 7: least       -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 8: Rank        -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 9 through 9+K: -   Design matrix where each row lists the effects 
%                           coded design variables for each alternative
%
%% Outputs
% The procedure saves a *.mat file that contains a structure called "data":
% Field                 Description
% data.alpha_rec        The recommended value for the prior weight
% data.RPS              An L x V matrix containing the root predictive 
%                       score values for each estimation run for L prior 
%                       weight values and for V validation subsamples
% data.RPS_rec          The average root predictive score value evaluated 
%                       at the recommended prior weight value
% data.RPS_bar          An L x 1 vector containing the average root 
%                       predictive score values for each prior weight value 
%                       tested
% data.RPS_bar_std      An L x 1 vector containing the sample standard 
%                       deviation of the root predictive scores for each 
%                       prior weight tested
% data.ModelParams_rec  A K x N matrix containing the estimates of the K 
%                       parameters for each of the N individuals in the 
%                       sample evaluated at the recommended prior weight
%
% The procedure also prints to the screen a figure showing the average root
% predictive score for each prior weight value tested (i.e., the values 
% from data.RPS_bar). The error bars correspond to the values from 
% data.RPS_bar_std.
% 
% 
%% NOTES
% 1. Data sets with a large number of individuals and large numbers of 
%    choice sets can take minutes to hours to evaluate. Also, the larger 
%    the number of prior weights L evaluated, the longer the computing 
%    time. You can test the setup prior to evaluating a large data set by 
%    specifying only a small number of individuals N, such as 5-10, in the 
%    'MyRunSettingsFile'. Additionally, it may be quicker to test a small 
%    number of prior weights in order to identify the most promising range 
%    of prior weights prior to specifying a large number of prior weights L
%    to evaluate.