%% Setup_CarInsurance
% function that provides the settings for the problem setup
% Written by Bart Frischknecht, first version 29 March 2013, 
% Edits:
%
%%%%%%%%%%%%%%%%%%%%%%%

%% INPUTS
% Variable          Dimension                       Units                   Description
% HO                scalar                          individuals             Number of people from total sample (N) to holdout from estimation. 
%                                                                           This variable does not refer to the individuals used in the validation 
%                                                                           subsamples. Instead, this number should be set to the desired number of respondents 
%                                                                           for a true holdout sample to be used for later testing. The data from the 
%                                                                           holdout individuals are excluded from all estimation and prior weight calibration
% dFF               1 x att vector                  -                       A vector that describes the model parameters. The number of entries corresponds 
%                                                                           to the number of attributes in the model. Each entry corresponds to the number 
%                                                                           of levels of the particular attribute. Attributes should be listed in the order 
%                                                                           they appear in the data
% ProdASC           scalar                          -                       ProdASC=1 if a parameter for "no choice" or "status quo" is to be estimated. This 
%                                                                           parameter should not be included in the dFF vector 
% J                 scalar                          alternatives            Number of alternatives per choice set faced by each individual
% T                 scalar                          -                       Number of holdout choice sets per person. This number should be set to the number 
%                                                                           of choice sets that are intended to be used for later testing. These choice sets 
%                                                                           are not included in any estimation or calibration procedure
% HoldoutAltSize    scalar                          -                       Total number of alternatives per individual that are included in the holdout choice sets
% DrawHoldOut       scalar                          -                       DrawHoldout=1 to select HO individuals for holdout sample randomly from the original sample  
%                                                                           DrawHoldout=0 to select HO individuals from the bottom of the original data file. 
%                                                                           The value of this parameter does not matter when HO=0
% ChoiceCol         scalar                          -                       Column in 'FirstChoiceData' that has the choice indicator ({0,1} with 1 indicating the 
%                                                                           chosen alternative)  
% Q                 scalar                          choice tasks            The number of market relevant choice tasks included in the notional sample when the 
%                                                                           market relevant prior is used
% ProbMR            (Q*J) x 1 vector                probability             The choice probability assigned to each alternative in each market relevant choice task
% MarkRelCS         (Q*J) x K matrix                -                       Coded design matrix for the market relevant choice sets where each row represents a 
%                                                                           particular alternative in a particular market relevant choice set
% Note: The following two parameters are legacy from a prior software
% implementation. These parameters should be left at the default values.
% SimData           scalar                          -                       To generate simulated data, SimData=1; Otherwise, estimation expects to load data file
% UtForm            scalar                          -                       The distribution of the utility function for simulated data
%
% 
%%%%%%%%%%%%%%%%%%%%%%%

%% OUTPUTS
%
% Params            scalar                          -                       The number of parameters to be estimated in the choice model
% InDesign_eff      matrix                          -                       Effects-coded design file
% FirstChoiceData   matrix                          -                       Name of the choice data including the design
% MyCLEstFile       handle                          -                       Pointer to individual conditional logit estimation file
%
%%%%%%%%%%%%%%%%%%%%%%%

function [HO, dFF, NameLabels, ProdASC, J, T, HoldoutAltSize, DrawHoldOut, ...
            ChoiceCol, Q, ProbMR, MarkRelCS, SimData, UtForm,...
            Params, FirstChoiceData, MyCLEstFile...
            ]=Setup_CarInsurance()
%%%%%%%%%% Car insurance profile %%%%%%%%%%

%% Inputs about respondents %%
HO=0;                               % Number of people from total sample (N) to holdout from estimation; this is before
                                    % the subsamples are taken. This possibility is ignored in this
                                    % version of the code, so H0=0; should be left unchanged.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Inputs about data %%
dFF=[4 4 2 2 2 2 2];                % Attributes and levels in the order that they are listed in the data
NameLabels={'Brand0';'Brand1';...
    'Brand2';'Fare0';'Fare1';...
    'Fare2';'Repair0';...
    'Flexi0';'Payment0';'Bonus0';...
    'Discount0'};                   % A K x 1 cell array of strings containing the names for each model parameter. Typically these 
                                    % correspond to an attribute and attribute level identification
ProdASC=0;                          % ProdASC=1 if a parameter for an alternative vs. "no choice" 
                                    % or "status quo" is to be estimated
J=4;                                % Number of alternatives per choice set
T=4;                                % Number of hold out choice sets per person
HoldoutAltSize=J*T;                 % Total number of alternatives in the holdout tasks
DrawHoldOut=1;                      % To Draw holdout from original data set

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Inputs about data format %%
ChoiceCol=5;                        % Column in the Choice data file that has the choice indicator variable


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Inputs for market relevant prior
%%%%%%%% NOTE: the market relevant choice set given here is fictional based
%%%%%%%% on a simulation study and does not correspond to the car insurance
%%%%%%%% context from which the data are taken

%These three inputs can be set to empty if the market relevant prior will
%not be used. 
Q=1;  % number of market relevant choice sets
%The format for the dependent variable ProbMR is to list the choice
%probabilities sequentially in a column vector for each alternative in each
%market relevant choice set
ProbMR=[    0.66335
            0.05176
            0.06386
            0.22103]; % Choice probabilities for the market relevant choice sets
%The format for the market relevant choice sets is the same as columns
%9,...,9+K in the choice data. That is each row represents an alternative
%and each column is a code for one of the variables describing the
%attributes of the alternatives
MarkRelCS=[0	0	1	-1	-1	-1	1	1	1	1	1
           0	0	1	0	1	0	-1	1	1	1	1
           0	0	1	0	1	0	1	-1	-1	1	-1
           -1	-1	-1	1	0	0	1	1	1	-1	1]; % This is the market relevant choice set;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Inputs about analysis procedure for simulated data %%
SimData=0; %To generate simulated data rather than load empirical data set 
           %SimData=1;
UtForm=2;   % For simulated data indicate the functional form of the 
            %simulated utility functions as available in the AssignPref
            %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Outputs
Params=sum(dFF,2)-size(dFF,2)+ProdASC;
%InDesign_eff=load('InCarInsuranceDesign_eff.txt'); %File that contains the effects-coded design
FirstChoiceData='FirstChoiceCarInsurance.txt';      %File that contains the choice data including the design
MyCLEstFile=@mxlmslICL;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Description
%These are the columns of the ChoiceData matrix. Each row represents
%one alternative from one choice set.  All the data from a single
%respondent are grouped together. If there is a product alternative specific constant, the 
%ProductASC is coded [1] for a product or choice alternative and coded
%[0] for the none or status quo option.  
%
%The first 8 columns should be the same in every data set. The columns
%beyond 8 should be the design columns where the alternatives have been
%appropriately coded

% Column 1: responseid  -   sequentially numbered respondents 1,...,N
% Column 2: subset      -   sequentially numbered choice tasks 1,...,S
% Column 3: alt         -   sequentially numbered alternatives 1,...,J	
% Column 4: holdout_indic   -   entry should equal 1 if the alternative is 
%                               to be witheld from estimation
% Column 5: choice      -   binary variable {0,1} with 1 indicating the 
%                           chosen alternative
% Column 6: most        -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 7: least       -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 8: Rank        -   the entries in this column do not effect the
%                           estimation and can contain any numerical value
% Column 9 through 9+K: -   Design matrix where each row lists the effects 
%                           coded design variables for each alternative 
% For car insurance data:
% Brand_EC0 -    9
% Brand_EC1 -    10
% Brand_EC2 -    11
% prem_EC0 -     12
% prem_EC1 -     13
% prem_EC2 -     14
% Repair_EC0 -   15
% Flexi_EC0 -    16
% Payment_EC0 -  17
% Bonus_EC0 -    18
% Discount_EC0 - 19